@@ -43,9 +43,9 @@ LIBSBML_CPP_NAMESPACE_BEGIN
  */
 SpeciesTypeComponentIndex::SpeciesTypeComponentIndex (unsigned int level, unsigned int version, unsigned int pkgVersion)
   : SBase(level, version)
-   ,mId ("")
+////   ,mId ("")
    ,mComponent ("")
-   ,mIdentifyingParent ("")
+////   ,mIdentifyingParent ("")
 {
   // set an SBMLNamespaces derived object of this package
   setSBMLNamespacesAndOwn(new MultiPkgNamespaces(level, version, pkgVersion));
@@ -57,9 +57,9 @@ SpeciesTypeComponentIndex::SpeciesTypeComponentIndex (unsigned int level, unsign
  */
 SpeciesTypeComponentIndex::SpeciesTypeComponentIndex (MultiPkgNamespaces* multins)
   : SBase(multins)
-   ,mId ("")
+////   ,mId ("")
    ,mComponent ("")
-   ,mIdentifyingParent ("")
+////   ,mIdentifyingParent ("")
 {
   // set the element namespace of this object
   setElementNamespace(multins->getURI());
@@ -74,9 +74,9 @@ SpeciesTypeComponentIndex::SpeciesTypeComponentIndex (MultiPkgNamespaces* multin
  */
 SpeciesTypeComponentIndex::SpeciesTypeComponentIndex (const SpeciesTypeComponentIndex& orig)
   : SBase(orig)
-  , mId  ( orig.mId)
+//  , mId  ( orig.mId)
   , mComponent  ( orig.mComponent)
-  , mIdentifyingParent  ( orig.mIdentifyingParent)
+//  , mIdentifyingParent  ( orig.mIdentifyingParent)
 {
 }
 
@@ -126,6 +126,16 @@ SpeciesTypeComponentIndex::getId() const
 }
 
 
+/*
+ * Returns the value of the "name" attribute of this SpeciesTypeComponentIndex.
+ */
+const std::string&
+SpeciesTypeComponentIndex::getName() const
+{
+  return mName;
+}
+
+
 /*
  * Returns the value of the "component" attribute of this SpeciesTypeComponentIndex.
  */
@@ -156,6 +166,16 @@ SpeciesTypeComponentIndex::isSetId() const
 }
 
 
+/*
+ * Returns true/false if name is set.
+ */
+bool
+SpeciesTypeComponentIndex::isSetName() const
+{
+  return (mName.empty() == false);
+}
+
+
 /*
  * Returns true/false if component is set.
  */
@@ -186,6 +206,17 @@ SpeciesTypeComponentIndex::setId(const std::string& id)
 }
 
 
+/*
+ * Sets name and returns value indicating success.
+ */
+int
+SpeciesTypeComponentIndex::setName(const std::string& name)
+{
+  mName = name;
+  return LIBSBML_OPERATION_SUCCESS;
+}
+
+
 /*
  * Sets component and returns value indicating success.
  */
@@ -241,6 +272,25 @@ SpeciesTypeComponentIndex::unsetId()
 }
 
 
+/*
+ * Unsets name and returns value indicating success.
+ */
+int
+SpeciesTypeComponentIndex::unsetName()
+{
+  mName.erase();
+
+  if (mName.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
 /*
  * Unsets component and returns value indicating success.
  */
@@ -352,7 +402,7 @@ SpeciesTypeComponentIndex::writeElements (XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -369,7 +419,7 @@ SpeciesTypeComponentIndex::accept (SBMLVisitor& v) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -384,7 +434,7 @@ SpeciesTypeComponentIndex::setSBMLDocument (SBMLDocument* d)
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -400,7 +450,7 @@ SpeciesTypeComponentIndex::enablePackageInternal(const std::string& pkgURI,
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -411,17 +461,18 @@ SpeciesTypeComponentIndex::enablePackageInternal(const std::string& pkgURI,
 SBase*
 SpeciesTypeComponentIndex::createObject(XMLInputStream& stream)
 {
-  const string& name = stream.peek().getName();
-  SBase* object = NULL;
+  return NULL;
+  //const string& name = stream.peek().getName();
+  //SBase* object = NULL;
 
-  MULTI_CREATE_NS(multins, getSBMLNamespaces());
+  //MULTI_CREATE_NS(multins, getSBMLNamespaces());
 
-  delete multins;
-  return object;
+  //delete multins;
+  //return object;
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -435,12 +486,13 @@ SpeciesTypeComponentIndex::addExpectedAttributes(ExpectedAttributes& attributes)
   SBase::addExpectedAttributes(attributes);
 
   attributes.add("id");
+  attributes.add("name");
   attributes.add("component");
   attributes.add("identifyingParent");
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -552,6 +604,22 @@ SpeciesTypeComponentIndex::readAttributes (const XMLAttributes& attributes,
                    getLine(), getColumn());
   }
 
+   //
+   // name string   ( use = "optional" )
+   //
+   assigned = attributes.readInto("name", mName);
+
+   if (assigned == true)
+   {
+     // check string is not empty
+
+     if (mName.empty() == true)
+     {
+       logEmptyString(mName, getLevel(), getVersion(), "<SpeciesTypeComponentIndex>");
+     }
+   }
+
+
   //
   // component SIdRef   ( use = "required" )
   //
@@ -606,7 +674,7 @@ SpeciesTypeComponentIndex::readAttributes (const XMLAttributes& attributes,
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -633,7 +701,7 @@ SpeciesTypeComponentIndex::writeAttributes (XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
 /*
@@ -797,7 +865,7 @@ ListOfSpeciesTypeComponentIndexes::createObject(XMLInputStream& stream)
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -825,7 +893,7 @@ ListOfSpeciesTypeComponentIndexes::writeXMLNS(XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
 /**
@@ -884,6 +952,20 @@ SpeciesTypeComponentIndex_getId(SpeciesTypeComponentIndex_t * stci)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+SpeciesTypeComponentIndex_getName(SpeciesTypeComponentIndex_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getName().empty() ? NULL : safe_strdup(cr->getName().c_str());
+}
+
+
 /**
  * 
  */
@@ -923,6 +1005,17 @@ SpeciesTypeComponentIndex_isSetId(SpeciesTypeComponentIndex_t * stci)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentIndex_isSetName(SpeciesTypeComponentIndex_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetName()) : 0;
+}
+
+
 /**
  * 
  */
@@ -956,6 +1049,17 @@ SpeciesTypeComponentIndex_setId(SpeciesTypeComponentIndex_t * stci, const char *
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentIndex_setName(SpeciesTypeComponentIndex_t * cr, const char * name)
+{
+  return (cr != NULL) ? cr->setName(name) : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
@@ -989,6 +1093,17 @@ SpeciesTypeComponentIndex_unsetId(SpeciesTypeComponentIndex_t * stci)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentIndex_unsetName(SpeciesTypeComponentIndex_t * cr)
+{
+  return (cr != NULL) ? cr->unsetName() : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
