@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -63,7 +63,7 @@ QualUniqueModelWideIds::~QualUniqueModelWideIds ()
  * Constraint.  Override the doCheck() method to define your own subset.
  */
 void
-QualUniqueModelWideIds::check_ (const Model& m, const Model& object)
+QualUniqueModelWideIds::check_ (const Model& m, const Model& )
 {
   doCheck(m);
 }
