@@ -2,36 +2,39 @@
  * @file    RenderInformationBase.cpp
  * @brief   abstract base class for local and global render information
  * @author  Ralph Gauges
+ * @author  Frank T. Bergmann
  *
- */
-/* Copyright 2010 Ralph Gauges
+ * <!--------------------------------------------------------------------------
+ * This file is part of libSBML.  Please visit http://sbml.org for more
+ * information about SBML, and the latest version of libSBML.
  *
- * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation.  A copy of the license agreement is
- * provided in the file named "LICENSE.txt" included with this software
- * distribution.  It is also available online at
- * http://sbml.org/software/libsbml/license.html
+ * Copyright (C) 2013-2017 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
- *
- * The original code contained here was initially developed by:
- *
- *     Ralph Gauges
+ * Copyright (C) 2011-2013 jointly by the following organizations: 
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *  
+ * Copyright 2010 Ralph Gauges
  *     Group for the modeling of biological processes 
  *     University of Heidelberg
  *     Im Neuenheimer Feld 267
  *     69120 Heidelberg
  *     Germany
  *
- *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
- *
- * Contributor(s):
- */
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.  A copy of the license agreement is provided
+ * in the file named "LICENSE.txt" included with this software distribution
+ * and also available online as http://sbml.org/software/libsbml/license.html
+ * ---------------------------------------------------------------------- -->*/
 
 #include "RenderInformationBase.h"
+
+#include <sbml/util/ElementFilter.h>
+
 #ifndef OMIT_DEPRECATED
 #ifdef DEPRECATION_WARNINGS
 #include <iostream>
@@ -52,9 +55,9 @@ LIBSBML_CPP_NAMESPACE_BEGIN
  */
 RenderInformationBase::RenderInformationBase (unsigned int level, unsigned int version, unsigned int pkgVersion)
   : SBase(level,version)
-  , mId("")
-  , mName ("")
-  , mProgramName ("")
+////  , mId("")
+////  , mName ("")
+////  , mProgramName ("")
   , mProgramVersion ("")
   , mReferenceRenderInformation ("")
   , mBackgroundColor("")
@@ -76,9 +79,9 @@ RenderInformationBase::RenderInformationBase (unsigned int level, unsigned int v
  */
 RenderInformationBase::RenderInformationBase (RenderPkgNamespaces* renderns)
   : SBase(renderns)
-  , mId("")
-  , mName ("")
-  , mProgramName ("")
+////  , mId("")
+////  , mName ("")
+////  , mProgramName ("")
   , mProgramVersion ("")
   , mReferenceRenderInformation ("")
   , mBackgroundColor("")
@@ -98,6 +101,14 @@ RenderInformationBase::RenderInformationBase (RenderPkgNamespaces* renderns)
 /** @endcond */
 
 
+/*
+ * Destroy this object.
+ */
+RenderInformationBase::~RenderInformationBase ()
+{
+}
+
+
 List*
 RenderInformationBase::getAllElements(ElementFilter* filter)
 {
@@ -213,8 +224,8 @@ void RenderInformationBase::parseXML(const XMLNode& node)
  */
 RenderInformationBase::RenderInformationBase(RenderPkgNamespaces* renderns, const std::string& id)
   : SBase(renderns)
-  , mId(id)
-  , mName ("")
+//  , mId(id)
+////  , mName ("")
   , mProgramName ("")
   , mProgramVersion ("")
   , mReferenceRenderInformation ("")
@@ -226,6 +237,7 @@ RenderInformationBase::RenderInformationBase(RenderPkgNamespaces* renderns, cons
 #ifdef DEPRECATION_WARNINGS
     std::cerr << "Warning. RenderInformationBase::RenderInformationBase(const std::string& id) is deprecated." << std::endl;
 #endif // DEPRECATION_WARNINGS
+    setId(id);
         // set the element namespace of this object
   setElementNamespace(renderns->getURI());
 
@@ -436,6 +448,7 @@ ColorDefinition* RenderInformationBase::createColorDefinition()
     {
         RENDER_CREATE_NS(renderns, this->getSBMLNamespaces());
         cd = new ColorDefinition(renderns);
+	   delete renderns;
     }
     catch (...)
     {
@@ -660,6 +673,7 @@ RadialGradient* RenderInformationBase::createRadialGradientDefinition()
     {
       RENDER_CREATE_NS(renderns, this->getSBMLNamespaces());
       rg = new RadialGradient(renderns);
+	 delete renderns;
     }
     catch (...)
     {
@@ -697,6 +711,7 @@ LinearGradient* RenderInformationBase::createLinearGradientDefinition()
     {
       RENDER_CREATE_NS(renderns, this->getSBMLNamespaces());
       lg = new LinearGradient(renderns);
+	 delete renderns;
     }
     catch (...)
     {
@@ -923,6 +938,7 @@ LineEnding* RenderInformationBase::createLineEnding()
     {
       RENDER_CREATE_NS(renderns, this->getSBMLNamespaces());
       le = new LineEnding(renderns);
+	 delete renderns;
     }
     catch (...)
     {
