@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2016 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -112,6 +112,8 @@ ArgumentsUnitsCheck::checkUnits (const Model& m, const ASTNode& node, const SBas
     case AST_RELATIONAL_LEQ:
     case AST_RELATIONAL_LT:
     case AST_RELATIONAL_NEQ:
+    case AST_FUNCTION_MAX:
+    case AST_FUNCTION_MIN:
   
       checkSameUnitsAsArgs(m, node, sb, inKL, reactNo);
       break;
